; -------------------------------------------------------------
; This procedure searches the environment for the currently
; installed Personal REXX Resident Manager and returns an appropriate result.
;
; Adapted from similar routine for PCED by Cove Software Group
;
; The procedure is designed to be used as an include file.  To use
; it, just INCLUDE this file in your program (or merge it in, if you
; wish) and execute a CALL to 'findrexx'.
;
; Entry assumption:
;    ES points to the program's PSP (which it will if you are creating
;    a standard COM or EXE program and ES contains its original value
;    from startup).
;
; Exit:
;    AL == 0:  Interrupt Manager is installed
;    AL == 1:  RXINT string in environment was invalid
;    AL == 2:  Some other program is using Interrupt Manager interrupt vector
;    AL == 3:  Interrupt Manager not installed
;
;    AH == Personal REXX Interrupt number (if AL != 1)
;
;    Other registers preserved
;
; NOTE 1: This routine was written without using any constants
; or other good programming stuff so that it can simply be
; included or merged into other programs intact, without
; needing any header files, etc.
;
; NOTE 2: All labels declared in this procedure begin with '$$G'
; to minimize collisions with your own labels.
; -------------------------------------------------------------
findrexx proc near

	push bx
	push cx
	push dx
	push si
	push di
	push ds

	mov dx,es			; Save ES
	mov ax,cs			; Point DS to code segment
	mov ds,ax

	mov es,es:[2CH] 		; ES = environment segment
	xor di,di			; Point to start of env in ES:DI
	cld

; ----- Loop thru environment strings one by one, beginning here
$$G1:
	test byte ptr es:[di],-1	; End of environment?
	jnz $$G1A			; Nope, continue
	mov al,60H			; Yes, use default interrupt
	jmp short $$G3

; ----- Compare current env string to 'RXINT='
$$G1A:
	mov si,offset RXINT$		; Point to 'RXINT=' string
	mov bx,di			; Save ptr to start of env string
	mov cx,6			; Length of 'RXINT='
	repe cmpsb			; Compare
	je $$G2 			; Found it!

	mov di,bx			; Restore ptr to start of env string
	xor al,al			; Scan for end of string
	mov cx,-1
	repne scasb
	jmp $$G1			; Go back for next string

; ----- Found it.  Get the interrupt number
$$G2:
	mov ax,es:[di]			; AX = int # (AL=hi byte)
	cmp byte ptr es:2[di],0 	; Should be 0 after int #
	jne $$G2A			; Oops

	call $$GX			; Convert AL from ASCII to binary
	jc $$G2A			; Error if carry
	mov cl,4			; Mov to high nibble
	shl al,cl
	xchg ah,al			; Put low byte in AL
	call $$GX			; Convert to binary
	jc $$G2A			; Error if carry
	or al,ah			; Merge the two nibbles
	cmp al,40H			; Only accept x'40' and up
	jae $$G3			; Jump if OK

; ----- Jump here on any error with the RXINT= string
$$G2A:
	mov al,1			; Return code 1
	jmp short $$G9

; ----- OK, AL has the purported Personal REXX interrupt number. Check it out.
$$G3:
	push ax 			; Save int number
	mov ah,35H			; Get ISR address to ES:BX
	int 21H
	mov di,bx			; ES:DI -> interrupt routine
	pop bx				; Recover int # (in BL)

	mov ax,es			; AX = interrupt routine segment
	or ax,di			; Merge offset; AX=0 if not installed
	jz $$G4 			; Indicate not installed

	add di,3			; ES:DI -> RXINT signature
	mov   si,offset RXINT$		; DS:SI -> what signature should be
	mov   cx,5			; Length of 'RXINT' (without '=')
	repe cmpsb			; Compare
	jne $$G5			; Uh-oh, someone else using vector

	mov al,0			; Signal all is well
	mov ah,bl			; AH = Personal REXX interrupt number
	jmp short $$G9

; ----- Here if Personal REXX interrupt vector not in use
$$G4:	mov al,3
	mov ah,bl			; AH = Personal REXX interrupt number
	jmp short $$G9

; ----- If here, someone else stole the vector
$$G5:
	mov al,2			; Return code 2
	mov ah,bl			; AH = Personal REXX interrupt number

; ----- Common exit.  AL has return code, AH has REXX interrupt number
$$G9:
	mov es,dx
	pop ds
	pop di
	pop si
	pop dx
	pop cx
	pop bx
	ret

; ----- Convert AL from ASCII to binary; return CF=1 if not a hex digit
$$GX:
	cmp al,'a'                      ; Ensure uppercase
	jb $$GX1
	cmp al,'z'
	ja $$GX1
	sub al,32
$$GX1:
	cmp al,'F'                      ; Error if > 'F'
	ja $$GXErr
	sub al,'0'                      ; First conversion
	jc $$GXErr			; Error if < '0'
	cmp al,9			; OK if <= 9
	jbe $$GX8
	sub al,7			; Second conversion
	cmp al,10			; Error if 2nd cnvrt results in < 10
	jb $$GXErr
$$GX8:	clc
$$GX9:	ret

$$GXErr:
	stc
	jmp $$GX9

RXINT$	db 'RXINT='                     ; String to search for in env
findrexx endp
