/*

This is MYAPPL.C, an example of REXX calls to an application environment.
Coded for Microsoft C version 5.1 small, medium, or large models.

Function:

    Establishes an application environment and runs a REXX program.
    Handles calls to the application environment from the REXX program.

Entry Conditions:

    This program should be run while REXX is loaded.
    Any parameters are ignored.

Include Files:

    REXXCALL.H   - maps REXX control block

Calls:

    FINDINT.ASM  - gets address of REXX interrupt handler
    REXXCALL.ASM - environment entry point, calls execute function.

Sample Usage:

    See the Personal REXX Calls to Application Environments section
    in the manual for a detailed example.

Notes:

    To recreate executable version of this program use the command

        MAKE APIMSC                     (small model)
        MAKE MODEL=-AM APIMSC           (medium model)
        MAKE MODEL=-AL APIMSC           (large model)

    MYAPPL.C uses C preprocessor statements to generate
    the correct code for any of the three models.

    FINDINT.ASM and REXXCALL.ASM also need no changes for the
    different models.

    This program was tested with Personal REXX 2.00, DOS 3.3, and MSC 5.1.

*/

#include <stdio.h>
#include <dos.h>
#include <malloc.h>

#include "rexxcall.h"

#define MAKE_LONG(hi, low) \
 ((long) ((((unsigned long) (unsigned) (hi)) << 16) + \
 (unsigned long) (unsigned) (low)))
#define HIGH_WORD(l) ((unsigned) (((unsigned long) (l)) >> 16))
#define LOW_WORD(l) ((unsigned) (((unsigned long) (l)) & 0xffff))

#ifdef M_I86SM  /* small model */
#define FINDINT findints
#endif
#ifdef M_I86MM  /* medium model */
#define FINDINT findintm
#endif
#ifdef M_I86LM  /* large model */
#define FINDINT findintl
#endif

unsigned showsp();

unsigned data_segment;

main(argc, argv)
 int argc;
 char **argv;
    {
    int retcode;

    /* call rexx interface routine, telling it to execute */
    /* rexx program 'MACRO', with argument 'foo args'       */
    proc("MACRO","FOO ARGS",&retcode);
    }

/* interface to rexx interpreter */
proc(name, argstring, rcode)
 char *name, *argstring;
 int *rcode;
    {
    int i;
    char *s, *t, result[257], fname[80];
    union REGS inregs, outregs;
    struct SREGS segregs;
    struct call_block cb, *cbp = &cb;
    struct arg_block arg_list;
    struct env_block eb;

    /* rexxcall is assembler routine called by rexx to setup  */
    /* segment registers, etc. so execute, defined later in this */
    /* module, can be called                */
    int rexxcall(), execute(),findint();

    int rexx_interrupt,interrupt_status;

    /* get REXX interrupt - abort if RXINTMGR is not installed */
    rexx_interrupt = FINDINT(&interrupt_status);
    if (interrupt_status != 0) error("RXINTMGR not installed.\n");

    /* insure REXX is loaded - abort if it isn't */
    inregs.h.ah = REXX_VERSION;
    inregs.h.al = 0;
    int86(rexx_interrupt, &inregs, &outregs);
    if (outregs.h.al == 0) error("REXX not loaded\n");

    /* fill in Call Block */
    segread(&segregs);
    data_segment = segregs.ds;

    memset(&cb, 0, sizeof(struct call_block));
    cb.arg_list_addr = (FARP) &arg_list;
    cb.arg_count = 1;
    cb.ret_size = 256;
    cb.ret_count = 0;
    cb.ret_addr = (FARP) result;
    cb.envblock_addr = (FARP) &eb;
    cb.name_addr = (FARP) name;
    cb.name_length = strlen(name);

    /* Fill in Argument Block */
    arg_list.arg_addr = (FARP) argstring;
    arg_list.arg_length = strlen(argstring);

    /* Fill in Environment Block */
    memset(&eb, 0, sizeof(struct env_block));
    memcpy(eb.env_name,"MYAPPL  ", 8);     /* name of our environment      */
    memcpy(eb.env_extension, "APL", 3);    /* extension of our macros      */
    eb.env_entry_addr = (FARP) rexxcall;   /* interface to command handler */
#ifdef M_I86SM   /* predefined by microsoft c if small model */
    /* 0 for segment address of execute will indicate to rexxcall */
    /* to use near call to execute              */
    eb.env_exec_addr = (FARP) MAKE_LONG(0, execute); /* command handler */
#else      /* medium or large model */
    eb.env_exec_addr = (FARP) execute;          /* command handler */
#endif
    eb.env_dseg = data_segment;
    eb.env_sseg = segregs.ss;
    /* make stack pointer put into effect when 'execute' is called          */
    /* start out a bit below current sp, nothing on our stack is overlayed */
    eb.env_sp = showsp() - 40;

    /* tell REXX to run the REXX program */
    inregs.h.ah = REXX_CALL;
#ifdef M_I86LM    /* predefined by microsoft c if large model */
    inregs.x.bx = FP_SEG(cbp);
    inregs.x.dx = FP_OFF(cbp);
#else       /* medium or small model */
    inregs.x.bx = data_segment;
    inregs.x.dx = (unsigned) cbp;
#endif
    int86x(rexx_interrupt, &inregs, &outregs, &segregs);

    result[cb.ret_count] = 0;  /* turn result into a c string */
    *rcode = atoi(result);
    }

#ifdef M_I86LM  /* large model */
/* returns the current stack pointer */
unsigned showsp()
    {
    int dummy, *dummyp = &dummy;

    return (FP_OFF(dummyp));
    }
#else     /* small and medium models */
/* returns the current stack pointer */
unsigned showsp ()
    {
    int dummy; /* dummy variable allocated on stack so we can get its address */

    return ((unsigned) &dummy);
    }
#endif

/* 'execute' a command */
/* this routine is called (via rexxcall assembler routine) when rexx */
/* encounters a command to be processed by our environment        */
int execute(s, l)
 FARP s;
 int l;
    {
    char *command;

    /* copy command string into a c-style null terminated string */
    command = malloc(l + 1);
#ifdef M_I86LM    /* large model */
    strncpy(command, s, l);
#else       /* small and medium models */
    movedata(HIGH_WORD(s), LOW_WORD(s),
     data_segment, command, l);
#endif
    command[l] = '\0';

    /* actually we just echo the command and give RC = -10 */
    printf("command: %s\n", command);
    return(-10);
    }

/* display error message and exit (similar to ci 86 abort() function) */
error(s)
char *s;
{
    printf("ABORT:- %s\n",s);
    exit(255);
}
